/*===============================================================
  Ethernet driver for Lucent WaveLAN based PC cards

  file: LucentWave.impl.h
  protocol class definitions

  Author: Hiroshi Noguchi

  03/04/2000	1st.
===============================================================*/




#ifndef __TDKETHERNET_IMPL_H
#define __TDKETHERNET_IMPL_H


#ifndef	__LANTERNDRIVER_PROTO_H
#include "LanternDriver.proto.h"
#endif

#ifndef	__LANTERNDRIVERAPI_PROTO_H
#include "LanternDriverAPI.proto.h"
#endif

#include "mydef.h"
#include "lucent_wi.h"




/*=======================================
  EthernetpPbgɊւ`
=======================================*/
#define kPacketAddrSize			(6)		/* Packet physical address size in byte */
#define kPacketTypeSize			(2)		/* Packet type size in byte */
#define kPacketCRCSize			(4)		/* Packet CRC size in byte */
#define kPacketHeaderSize		(14)	/* Packet header size in byte */

typedef UINT8	TEtherAddr[kPacketAddrSize];

typedef struct {
	TEtherAddr	fDestinationAddr;	/* Destination address */
	TEtherAddr	fSourceAddr;		/* Source address */
	UINT16		fLengthType;		/* Length/type */
} TPacketHeader;

#define kPacketMaxDataLength	(1500)			/* Max packet data length */
#define kPacketMinDataLength	(46)			/* Min packet data length */

#define kPacketTxMaxLength	(kPacketHeaderSize + kPacketMaxDataLength)	/* 1514	Max Tx packet length */
#define kPacketTxMinLength	(kPacketHeaderSize + kPacketMinDataLength)	/* 60	Min Tx packet length */

#define kPacketRxMaxLength	kPacketTxMaxLength
#define kPacketRxMinLength	kPacketTxMinLength



typedef struct {
	INT16	bytes;
	UINT16	dummy;
	UINT8	buf[1536];
} TPacketBuf;



class CList;
class CBufferList;
class EventTraceCauseDesc;
class TEventCollector;
class TMulticastAddrDataBase;



/*================================================
  vgRNX `
================================================*/
PROTOCOL TLucentWaveLAN : public TLanternDriver	/* Protocol class */
	PROTOCOLVERSION(1.0)						/* Protocol class version */
{
  public:
	/***********************************
	  vgRNX C^[tF[X
	***********************************/
	PROTOCOL_IMPL_HEADER_MACRO(TLucentWaveLAN);	/* Magic stuff, ΊO!! (for ProtocolGen) */

	TLanternDriver*	New(void);		/* Protocol class constructor, called when the card is inserted */
	void			Delete();		/* Protocol class destructor, called when the card is removed */

	/***********************************
	  ^XNT[rX
	***********************************/
	NewtonErr	Init(void);				/* Driver initialization */
	NewtonErr	Enable();				/* Start/resume card operations (startup/newton power on) */
	NewtonErr	Disable();				/* Stop/suspend card operations (shutdown/newton power off) */
	void		InterruptHandler();		/* Called to handle a card interrupt */

  private:
	friend		class	TMulticastAddrDataBase;

	/*
	 * NCAgT[rX[`
	 * TLanternDriverAPĨCxgnh
	 * Ăяo
	 */
	void		SendBuffer(Ptr thePacket, Size size);		/* Transmit a buffer */
	void		SendCBufferList(CBufferList* thePacket);	/* Transmit a CBufferList */
	NewtonErr	SendPacket(UChar*, CBufferList*, ULong);	/* pPbgo */
	void		GetDeviceAddress(UByte* addr, Size size);	/* Return the device address */
	void		AddMulticastAddress(UByte* addr);			/* Add a multicast address */
	void		DelMulticastAddress(UByte* addr);			/* Delete a multicast address */
	void		GetLinkIntegrity();							/* Get the link integrity flag */
	void		SetPromiscuous(ULong prom);			/* Set promiscuous mode (prom <> 0: enable) */
	void		GetThroughput();					/* Get the driver throughput if available */

	/*
	 * hCoƎ̊OCxgnh
	 */
	void		SetConfig( ULong, ULong, void* );
	void		GetConfig( ULong );

	/*
	 * IȃCxgnh
	 */
	void		TimerExpired(void);						/* Called when driver timer expires */
	void		TimerProcBeat(void);
		
	NewtonErr	InitCard();								/* J[h */

	/*
	 * n[hEFAˑ̃[`
	 */
	UINT16		NICReadRegW( UINT32 );
	void		NICWriteRegW( UINT32, UINT16 );
	INT32		NICCmd( UINT16, UINT16 );
	INT32		NICSeek( INT32, INT32, INT32 );
	INT32		NICReadRecord( struct wi_ltv_gen* );
	INT32		NICWriteRecord( struct wi_ltv_gen* );
	void		NICSetVal( UINT16, UINT16 );
	void		NICSetStr( UINT16, UINT8* );
	UINT16		NICGetVal( UINT16 );
	INT32		NICReadData( INT32, INT32, void*, INT32 );
	INT32		NICWriteData( INT32, INT32, void*, INT32 );
	INT32		NICReset();
	INT32		NICAllocNICMem( INT32, INT32* );
	INT32		NICUpdateStatus();
	void		NICGetID();

	void		NICInitVars();							/* ݒp[^  */
	NewtonErr	NICInit();								/* NIC  */
	NewtonErr	NICEnable();							/* M Jn */
	NewtonErr	NICDisable();							/* M ~ */

	void		NICInterrupt();							/* 荞݉ */
	NewtonErr	NICTxInt( UINT16 );						/* M荞 */
	NewtonErr	NICRxInt();								/* M荞 */

	NewtonErr	NICWritePacket( UINT32,
							    UINT8*, CBufferList* );	/* pPbg */
	NewtonErr	NICXmit();								/* MN */

	void		NICSetMulti();
	NewtonErr	NICAddDeleteMulticastAddr(UChar*, Boolean );	/* }`LXgAhX ǉ/폜 */
	Boolean		NICGetLinkIntegrity();					/* Get link integrity */
	NewtonErr	NICSetPromiscuous(ULong prom);			/* Set promiscuous mode (prom <> 0: enable) */

	/*
	 * ϐ
	 */
	TMulticastAddrDataBase*	fMulticastAddrDataBase;		/* Multicast address data base */

	ULong		fCardConfigBaseAddr;					/* Card configuration reg base address */
	UChar		fCardConfigNumber;						/* Card configuration number */
	Boolean		fFlagCard3VAvail;						/* whether 3.3V available */
	ULong		fNICBaseAddr;							/* NIC base address */

	ULong		fStatus;								/* Internal status */
	TEtherAddr	fEthernetAddr;							/* Ethernet address */
	Boolean		fLinkIntegrity;							/* Link integrity status */

	Boolean		fTimerPosted;							/* Is a timer event posted. */
	TTime		fTickTime;								/* TIme for the next timer event */

	/* ŗLɕKvȃ^C}[ */
	Boolean		fFlagBeatTimer;
	TTime		fTimeBeat;

	/*
	 * LAN ݒ
	 */
	UINT16		fPortNum;
	UINT16		fPortType;		/* ʐM[h */
	UINT16		fApDensity;
	UINT16		fRTSThresh;
	UINT16		fTxRate;
	UINT16		fMaxDataLen;
	UINT16		fCreateIBSS;	/* IBSS ON/OFF */
	UINT16		fPMEnabled;
	UINT16		fMaxSleep;
	UINT16		fOwnChannel;	/* g`lݒ */
	UINT16		fRoaming;
	UINT16		fAuthType;
	UINT16				fHasWEP;	/* WEP@\̗L */
	UINT16				fUseWEP;	/* WEP@\ ON/OFF */
	UINT8				fWEPKeySel;
	struct wi_ltv_keys	fWEPKeys;
									/*
									 *  64bits -->  40bits =  5 bytes
									 * 128bits --> 104bits = 13 bytes
									 */
	INT8		fNodeName[32];
	INT8		fNetName[32];			/* lbg[N (infrastructure mode) */
	INT8		fIBSSName[32];			/* lbg[N (802.11 ad-hoc mode) */

	INT32		fFirmwareType;			/* t@[EFA */
	INT32		fFirmwareVerPrimary;
	INT32		fFirmwareVerStation;

	/*--- Ǝ ---*/
	UINT16		fWaveLANMode;			/* ʐM[h */
	INT32		fFlagRegistOK;			/* o^ς݂ł邩tO */

	/*
	 * n[hEFAp
	 */
	INT32		fIDTxBuf;
	wi_counters	fCounters;		/* statistics */

	/* Mobt@Ǘp */
#define	TX_PKT_NUM	(1)
	TPacketBuf	fTxPktBufArray[TX_PKT_NUM];
	CList*		fPtrListTxBufPool;
	CList*		fPtrListTxBufQueue;

	/* fobOp */
	INT32		fDebugLevel;
#ifdef forDebug
	UINT32					fDebugSignature;
	UINT32					fDebugControls;
	EventTraceCauseDesc*	fEventTraceCauseDescs;			/* Event trace cause descriptors */
	TEventCollector*		fEventTraceCollector;			/* Event trace collector */
#endif
};



/*================================================
  Xe[^X`
================================================*/
enum {
	DRV_STAT_NONE			= 0x0L,

	DRV_STAT_CARD_POWER_ON	= K_BIT4,	/* J[hʓdĂ */
	DRV_STAT_ENABLED		= K_BIT5,	/* M\ł邱Ƃ */
	DRV_STAT_TX_BUSY		= K_BIT6,	/* Mbusy?  */
	DRV_STAT_RX_PROMISCUOUS	= K_BIT7	/* SpPbgM[h */
};


/*
 *	Run time debugging controls for selecting different actions
 *  at run time without recompile the code
 */
enum kDebugControls
{
	kDebugControlNoBroadcastPackets		=	0x00010000,
	kDebugControlAcceptAllPackets		=	0x00020000,
	kDebugControlSaveErrorPackets		=	0x00040000,
	kDebugControlRxMonitor				=	0x00080000,

	kDebugControlBreakOnRxTxErrors		=	0x00100000,
	kDebugControlPrintProgress			=	0x00200000,

	kDebugControlTestRx					=	0x01000000,
	kDebugControlPrintRxPackets			=	0x02000000,

	kDebugControlTestTx					=	0x10000000,
	kDebugControlPrintTxQueue			=	0x20000000
};



/*---------------------------------
  }`LXgAhX f[^x[X
  ǗNX
  "TMulticastAddrDataBase"
---------------------------------*/
/*
 * The multicast address data base keeps
 * track the number of requests for each address.
 * When the request count of a multicast address reaches 0,
 * the address is removed from NIC chip.
 */

/*
 * TMulticastAddrEntry
 * }`LXgAhX Gg\
 */
struct TMulticastAddrEntry
{
	TEtherAddr	fMulticastAddr;		/* }`LXgAhX */
	UINT8		fRequestCount;		/* QƃJEg */
};


/*
 * TMulticastAddrDataBase
 * }`LXgAhX f[^x[X NX
 */
class TMulticastAddrDataBase
{
  public:
	TMulticastAddrDataBase();
	~TMulticastAddrDataBase();

	/* Add or delete a multicast address and return current request count */
	NewtonErr				AddDeleteMulticastAddr(UChar* addr, Boolean add, Boolean* updateNIC);

	/* Find the multicast addresss in list */
	TMulticastAddrEntry*	FindMulticastAddr(UChar* addr);

	NewtonErr				GetAddrList( TEtherAddr[], INT32* );

  private:
	CList					fMulticastAddressList;		// TMulticastAddrEntry list
};




#endif	/* __TDKETHERNET_IMPL_H */
/*----------------
  end of file
----------------*/
